@@ -5,7 +5,7 @@ from pythoncom import com_error
 import robot.libraries.Screenshot as screenshot
 import os
 from robot.api import logger
-from subprocess import check_output,call
+from subprocess import check_output,Popen
 
 class AISAPLibrary:
     """The SapGuiLibrary is a library that enables users to create tests for the Sap Gui application
@@ -799,7 +799,38 @@ class AISAPLibrary:
             self.click_element('wnd[1]/usr/btnSPOP-OPTION1')
         except:
             pass
-    
+        
+    def shell_content_dynamic_selection(self,shell_element=None,interactions='expand',node_num='1'):
+        ''' Script untuk interaksi dengan sheell element pada SAP GUI pada dynamic selection
+        ada beberapa interaksi yang dapat dilakukan yaitu select, top, expand dan double klik     
+        '''
+        try:
+            if interactions=='top':
+                self.session.findById(shell_element).topNode = "         "+node_num
+            if interactions=='select':
+                self.session.findById(shell_element).selectNode("         "+node_num)
+            if interactions=='expand':
+                self.session.findById(shell_element).expandNode("         "+node_num)                
+            if interactions=='doubleclick':
+                self.session.findById(shell_element).doubleClickNode("         "+node_num)
+        except Exception as e:
+            raise RuntimeError(e)
+
+    def shell_content_element(self,shell_element=None,interactions='get',row_num=1,col_name=None):
+        ''' Script untuk interaksi dengan sheell element pada SAP GUI, terutama untuk pada tabel data,
+        ada dua interaksi yang bisa dilakukan yaitu get (mengambil data, text value) dan click (melakukan double klik pada sel terpilih)    
+        '''
+        try:
+            if interactions=='get':
+                return self.session.findById(shell_element).GetCellValue(row_num,col_name)
+            if interactions=='click':
+                click = self.session.findById(shell_element)
+                click.SetCurrentCell(row_num,col_name)
+                click.DoubleClickCurrentCell()
+                return "Cell Clicked"
+        except Exception as e:
+            raise RuntimeError(e)
+        
     def login_sap(self, config):
         """ Script untuk Login SAP
             mengembalikan variabel status (boolean) dan message (string)
@@ -810,26 +841,36 @@ class AISAPLibrary:
             
             contoh penggunaan :
             
-                ${status}    ${msg}=    Login Sap    ${config}
+                ${msg}=    Login Sap    ${config}
             
             """
             
-        call([r'C:/Program Files (x86)/SAP/FrontEnd/SAPgui/saplogon.exe'])
+        Popen([r'C:/Program Files (x86)/SAP/FrontEnd/SAPgui/saplogon.exe'])
+        
+        # for i in range(10):
+        #     s = check_output('tasklist', shell=True)
+        #     if "saplogon.exe" in str(s):
+        #         break
+        #     else :
+        #         time.sleep(2)
         
         for i in range(10):
-            s = check_output('tasklist', shell=True)
-            if "saplogon.exe" in str(s):
+            try:
+                self.connect_to_session()
                 break
-            else :
+            except :
                 time.sleep(2)
         
-        self.connect_to_session()
-                
+        if self.sapapp == -1:
+            raise RuntimeError('Cannot connect to session')
+
         self.open_connection(config['connection'])
         self.input_text("wnd[0]/usr/txtRSYST-BNAME",config["username"])
         self.input_password("wnd[0]/usr/pwdRSYST-BCODE",config["password"])
         self.input_text("wnd[0]/usr/txtRSYST-MANDT",config['client'])
         self.send_vkey(0)
+
+        time.sleep(5)
         
         
         msg = "Success Login"
